@@ -1,11 +1,15 @@
-### tk2widgets.R - Support for the tile Tk widgets
+### tk2widgets.R - Support for the ttk widgets
 ### Copyright (c), Philippe Grosjean (phgrosjean@sciviews.org)
 ### Licensed under LGPL 3 or above
 ###
 ### Changes:
+### - 2012-01-09: reworked tk2label to provide "full options"
+###
+### - 2012-01-07: listbox now behaves like a ttk widget, although it is not!
+###
 ### - 2009-06-30: only use ttk (no tile or plain widgets)
 ###
-### - 2007-01-01: fisrt version (for tcltk2_1.0-0)
+### - 2007-01-01: first version (for tcltk2_1.0-0)
 ###
 ### To do:
 ### - Rework all this and add new widgets like sizegrip, tkplot, ...
@@ -68,12 +72,37 @@ tk2frame <- function (parent, ...)
 	return(w)
 }
 
-tk2label <- function (parent, tip = "", ...)
+tk2label <- function (parent, tip, label, tag, cfglist, wrap = FALSE, ...)
 {
+	## Also image, text, textvariable, label & tag for data
+	## width = -10, compound = "left", justify = "left", wrap = FALSE for config
+	## Special treatment from wrap (wraplength)
 	if (!is.ttk()) stop("Tcl/Tk >= 8.5 is required")
+	## If a config list is provided, rework arguments
+	if (!missing(cfglist)) {
+		args <- .mergeList(cfglist, list(...))
+		args$parent <- parent
+		if (!missing(tip)) args$tip <- tip
+		if (!missing(label)) args$label <- label
+		if (!missing(tag)) args$tag <- tag
+		return(do.call(tk2label, args))
+	}
+	## Create the widget and the corresponding R object
 	w <- tkwidget(parent, "ttk::label", ...)
-	if (tip != "") tk2tip(w, tip)
 	class(w) <- c("tk2label", "tk2widget", class(w))
+	if (!missing(tip)) tip(w) <- tip
+	if (!missing(label)) label(w) <- label
+	if (!missing(tag)) tag(w) <- tag
+	## Special treatment for 'wrap' argument that does not exists in ttk::label
+	if (!missing(wrap)) {
+		## We need width
+		width <- abs(as.integer(tclvalue(tkcget(w, "-width"))))
+		if (isTRUE(wrap)) wraplength <- .wraplength(w, width) else wraplength <- 0
+		tkconfigure(w, wraplength = wraplength)
+		## If width is not reapplied after wraplength, the text is not always
+		## wrapped in the widget (is this a bug?)
+		if (wraplength > 0 && length(width)) tkconfigure(w, width = width)
+	}	
 	return(w)
 }
 
@@ -85,18 +114,111 @@ tk2labelframe <- function (parent, ...)
 	return(w)
 }
 
-tk2listbox <- function (parent, selectmode = c("single", "browse", "multiple", "extended"),
-tip = "", ...)
+## TODO: I need to rework this on the same scheme as tk2label
+tk2listbox <- function (parent, values, value, selection,
+selectmode = c("extended", "single", "browse", "multiple"), height = 5,
+tip = "", scroll = "both", autoscroll = "x", enabled = TRUE, ...)
 {
-### TODO: associate list and selection with a variable and configure the list
-### TODO: autohiding scrollbars
+	## Check conditions and arguments
 	if (!is.ttk()) stop("Tcl/Tk >= 8.5 is required")
-	selectmode <- as.character(selectmode[1])
-	w <- tkwidget(parent, "listbox", font = "TkDefaultFont",
-		borderwidth = 0, selectmode = selectmode, exportselection = 0, ...)
+	selectmode <- match.arg(selectmode)
+	scrolls <- c("x", "y", "both", "none")
+	scroll <- match.arg(scroll, scrolls)
+	autoscroll <- match.arg(autoscroll, scrolls)
+	if (missing(values) || length(values) == 0) {
+		values <- NULL
+	} else {
+		values <- as.character(values)
+	}
+	## We provide either value or selection... translate value into selection
+	if (missing(selection)) selection <- NULL
+	if (!missing(value)) {
+		if (is.null(values) || length(values) < 1) selection <- NULL else
+			selection <- (1:length(values))[values %in% as.character(value)]
+	}
+	if (length(selection) == 0 || is.null(values) || length(values) < 1) {
+		selection <- NULL
+	} else {
+		selection <- sort(as.integer(round(selection)))
+		if (selection[1] < 1)
+			stop("Numerical selections must be indices > 0")
+		if (selection[1] > length(values)) selection <- NULL
+		if (selectmode == "single" && length(selection) > 1) {
+			warning("Only lowest selection used in single selection mode")
+			selection <- selection[1]
+		}
+	}
+	
+	## Location of the widget depends if we add scrollbars or not
+	if (scroll == "none") {
+		w <- tkwidget(parent, "listbox", font = "TkDefaultFont",
+			borderwidth = 1, relief = "sunken", activestyle = "dotbox",
+			selectmode = selectmode, height = height, exportselection = 0,
+			background = "#ffffff", ...)
+	} else { # We need to create a tk2frame as parent of the listbox
+		wf <- tk2frame(parent)
+		w <- tkwidget(wf, "listbox", font = "TkDefaultFont",
+			borderwidth = 1, relief = "sunken", activestyle = "dotbox",
+			selectmode = selectmode, height = height, exportselection = 0,
+			background = "#ffffff", ...)
+	}
+	## Make it react to tk2theme changes, and integrate the listbox as much
+	## as possible with current ttk theme
+	restyleListbox <- function (W) {
+		## Restyle the listbox according to current ttk style
+		## Note: font is set to TkDefaultFont => already managed there!
+		tkconfigure(W,
+			foreground = tk2style("tk2entry", "foreground",
+				default = "#000000"),
+			borderwidth = tk2style("", "borderwidth", default = 0),
+			disabledforeground = tk2style("tk2entry", "foreground",
+				"disabled", default = "#a3a3a3"),
+			highlightbackground = tk2style("tk2entry", "selectbackground",
+				default = "#c3c3c3"),
+			highlightcolor = tk2style("tk2entry", "selectbackground",
+				default = "#c3c3c3"),
+			selectbackground = tk2style("tk2entry", "selectbackground",
+				default = "#c3c3c3"),
+			selectforeground = tk2style("tk2entry", "selectforeground",
+				default = "#ffffff")
+		)
+	}
+	## Restyle it now
+	restyleListbox(w)
+	
+	## If there are values and/or selections, populate the list now
+	for (item in values)
+		tkinsert(w, "end", item)
+	if (!is.null(selection)) {
+		for (sel in selection)
+			tkselection.set(w, sel - 1) # Because Tcl uses 0-based indexing!
+		tksee(w, selection[1]) # Ensure that the first selected item is visible	
+	}
+	
+	## Possibly add a tooltip
 	if (tip != "") tk2tip(w, tip)
-	class(w) <- c("tk2listbox", "tk2widget", class(w))
-	return(w)
+	
+	## Do we add scrollbars?
+	if (scroll == "none") {
+		## Apply bindings to original listbox
+		tkbind(w, "<<ThemeChanged>>", restyleListbox)
+		tkbind(w, "<1>", function (W) tkfocus(W)) # Needed for mouseweel action
+		## Do we disable it?
+		if (!isTRUE(enabled)) tkconfigure(w, state = "disabled")
+		## Done... just return the widget
+		class(w) <- c("tk2listbox", "tk2widget", class(w))
+		return(w)
+	} else {
+		## Add (autohide) scrollbar(s)
+		tcl("scrolledWidget", w, wf, scroll, autoscroll)
+		## Apply bindings to frame container
+		tkbind(wf, "<<ThemeChanged>>", restyleListbox)
+		tkbind(wf, "<1>", function (W) tkfocus(W)) # Needed for mouseweel action
+		class(wf) <- c("tk2listbox", "tk2widget", class(w))
+		## Do we disable it?
+		if (!isTRUE(enabled)) tkconfigure(wf, state = "disabled")
+		return(wf)
+	}
 }
 
 tk2mclistbox <- function (parent, tip ="", ...)
